@@ -500,8 +500,273 @@ dc1394_bayer_HQLinear(const uint8_t *restrict bayer, uint8_t *restrict rgb, int
 dc1394error_t
 dc1394_bayer_EdgeSense(const uint8_t *restrict bayer, uint8_t *restrict rgb, int sx, int sy, int tile)
 {
-    /* Removed due to patent concerns */
-    return DC1394_FUNCTION_NOT_SUPPORTED;
+	uint8_t *outR, *outG, *outB;
+    register int i3, j3, base;
+    int i, j;
+    int dh, dv;
+    int tmp;
+	int sx3=sx*3;
+
+    // sx and sy should be even
+    switch (tile) {
+    case DC1394_COLOR_FILTER_GRBG:
+    case DC1394_COLOR_FILTER_BGGR:
+        outR = &rgb[0];
+        outG = &rgb[1];
+        outB = &rgb[2];
+        break;
+    case DC1394_COLOR_FILTER_GBRG:
+    case DC1394_COLOR_FILTER_RGGB:
+        outR = &rgb[2];
+        outG = &rgb[1];
+        outB = &rgb[0];
+        break;
+    default:
+		return DC1394_INVALID_COLOR_FILTER;
+    }
+
+    switch (tile) {
+    case DC1394_COLOR_FILTER_GRBG:        //---------------------------------------------------------
+    case DC1394_COLOR_FILTER_GBRG:
+        // copy original RGB data to output images
+		for (i = 0, i3=0; i < sy*sx; i += (sx<<1), i3 += (sx3<<1)) {
+			for (j = 0, j3=0; j < sx; j += 2, j3+=6) {
+				base=i3+j3;
+				outG[base]           = bayer[i + j];
+				outG[base + sx3 + 3] = bayer[i + j + sx + 1];
+				outR[base + 3]       = bayer[i + j + 1];
+				outB[base + sx3]     = bayer[i + j + sx];
+			}
+		}
+		// process GREEN channel
+		for (i3= 3*sx3; i3 < (sy - 2)*sx3; i3 += (sx3<<1)) {
+			for (j3=6; j3 < sx3 - 9; j3+=6) {
+				base=i3+j3;
+				dh = abs(((outB[base - 6] +
+						   outB[base + 6]) >> 1) -
+						   outB[base]);
+				dv = abs(((outB[base - (sx3<<1)] +
+						   outB[base + (sx3<<1)]) >> 1) -
+						   outB[base]);
+				tmp = (((outG[base - 3]   + outG[base + 3]) >> 1) * (dh<=dv) +
+					   ((outG[base - sx3] + outG[base + sx3]) >> 1) * (dh>dv));
+				//tmp = (dh==dv) ? tmp>>1 : tmp;
+				CLIP(tmp, outG[base]);
+			}
+		}
+		
+		for (i3=2*sx3; i3 < (sy - 3)*sx3; i3 += (sx3<<1)) {
+			for (j3=9; j3 < sx3 - 6; j3+=6) {
+				base=i3+j3;
+				dh = abs(((outR[base - 6] +
+						   outR[base + 6]) >>1 ) -
+						   outR[base]);
+				dv = abs(((outR[base - (sx3<<1)] +
+						   outR[base + (sx3<<1)]) >>1 ) -
+						   outR[base]);
+				tmp = (((outG[base - 3]   + outG[base + 3]) >> 1) * (dh<=dv) +
+					   ((outG[base - sx3] + outG[base + sx3]) >> 1) * (dh>dv));
+				//tmp = (dh==dv) ? tmp>>1 : tmp;
+				CLIP(tmp, outG[base]);
+			}
+		}
+		// process RED channel
+		for (i3=0; i3 < (sy - 1)*sx3; i3 += (sx3<<1)) {
+			for (j3=6; j3 < sx3 - 3; j3+=6) {
+				base=i3+j3;
+				tmp = outG[base] +
+					((outR[base - 3] -
+					  outG[base - 3] +
+					  outR[base + 3] -
+					  outG[base + 3]) >> 1);
+				CLIP(tmp, outR[base]);
+			}
+		}
+		for (i3=sx3; i3 < (sy - 2)*sx3; i3 += (sx3<<1)) {
+			for (j3=3; j3 < sx3; j3+=6) {
+				base=i3+j3;
+				tmp = outG[base] +
+					((outR[base - sx3] -
+					  outG[base - sx3] +
+					  outR[base + sx3] -
+					  outG[base + sx3]) >> 1);
+				CLIP(tmp, outR[base]);
+			}
+			for (j3=6; j3 < sx3 - 3; j3+=6) {
+				base=i3+j3;
+				tmp = outG[base] +
+					((outR[base - sx3 - 3] -
+					  outG[base - sx3 - 3] +
+					  outR[base - sx3 + 3] -
+					  outG[base - sx3 + 3] +
+					  outR[base + sx3 - 3] -
+					  outG[base + sx3 - 3] +
+					  outR[base + sx3 + 3] -
+					  outG[base + sx3 + 3]) >> 2);
+				CLIP(tmp, outR[base]);
+			}
+		}
+
+		// process BLUE channel
+		for (i3=sx3; i3 < sy*sx3; i3 += (sx3<<1)) {
+			for (j3=3; j3 < sx3 - 6; j3+=6) {
+				base=i3+j3;
+				tmp = outG[base] +
+					((outB[base - 3] -
+					  outG[base - 3] +
+					  outB[base + 3] -
+					  outG[base + 3]) >> 1);
+				CLIP(tmp, outB[base]);
+			}
+		}
+		for (i3=2*sx3; i3 < (sy - 1)*sx3; i3 += (sx3<<1)) {
+			for (j3=0; j3 < sx3 - 3; j3+=6) {
+				base=i3+j3;
+				tmp = outG[base] +
+					((outB[base - sx3] -
+					  outG[base - sx3] +
+					  outB[base + sx3] -
+					  outG[base + sx3]) >> 1);
+				CLIP(tmp, outB[base]);
+			}
+			for (j3=3; j3 < sx3 - 6; j3+=6) {
+				base=i3+j3;
+				tmp = outG[base] +
+					((outB[base - sx3 - 3] -
+					  outG[base - sx3 - 3] +
+					  outB[base - sx3 + 3] -
+					  outG[base - sx3 + 3] +
+					  outB[base + sx3 - 3] -
+					  outG[base + sx3 - 3] +
+					  outB[base + sx3 + 3] -
+					  outG[base + sx3 + 3]) >> 2);
+				CLIP(tmp, outB[base]);
+			}
+		}
+		break;
+
+    case DC1394_COLOR_FILTER_BGGR:        //---------------------------------------------------------
+    case DC1394_COLOR_FILTER_RGGB:
+        // copy original RGB data to output images
+		for (i = 0, i3=0; i < sy*sx; i += (sx<<1), i3 += (sx3<<1)) {
+			for (j = 0, j3=0; j < sx; j += 2, j3+=6) {
+				base=i3+j3;
+				outB[base] = bayer[i + j];
+				outR[base + sx3 + 3] = bayer[i + sx + (j + 1)];
+				outG[base + 3] = bayer[i + j + 1];
+				outG[base + sx3] = bayer[i + sx + j];
+			}
+		}
+		// process GREEN channel
+		for (i3=2*sx3; i3 < (sy - 2)*sx3; i3 += (sx3<<1)) {
+			for (j3=6; j3 < sx3 - 9; j3+=6) {
+				base=i3+j3;
+				dh = abs(((outB[base - 6] +
+						   outB[base + 6]) >> 1) -
+						   outB[base]);
+				dv = abs(((outB[base - (sx3<<1)] +
+						   outB[base + (sx3<<1)]) >> 1) -
+						   outB[base]);
+				tmp = (((outG[base - 3]   + outG[base + 3]) >> 1) * (dh<=dv) +
+					   ((outG[base - sx3] + outG[base + sx3]) >> 1) * (dh>dv));
+				//tmp = (dh==dv) ? tmp>>1 : tmp;
+				CLIP(tmp, outG[base]);
+			}
+		}
+		for (i3=3*sx3; i3 < (sy - 3)*sx3; i3 += (sx3<<1)) {
+			for (j3=9; j3 < sx3 - 6; j3+=6) {
+				base=i3+j3;
+				dh = abs(((outR[base - 6] +
+						   outR[base + 6]) >> 1) -
+						   outR[base]);
+				dv = abs(((outR[base - (sx3<<1)] +
+						   outR[base + (sx3<<1)]) >> 1) -
+						   outR[base]);
+				tmp = (((outG[base - 3]   + outG[base + 3]) >> 1) * (dh<=dv) +
+					   ((outG[base - sx3] + outG[base + sx3]) >> 1) * (dh>dv));
+				//tmp = (dh==dv) ? tmp>>1 : tmp;
+				CLIP(tmp, outG[base]);
+			}
+		}
+		// process RED channel
+		for (i3=sx3; i3 < (sy - 1)*sx3; i3 += (sx3<<1)) {        // G-points (1/2)
+			for (j3=6; j3 < sx3 - 3; j3+=6) {
+				base=i3+j3;
+				tmp = outG[base] +
+					((outR[base - 3] -
+					  outG[base - 3] +
+					  outR[base + 3] -
+					  outG[base + 3]) >>1);
+				CLIP(tmp, outR[base]);
+			}
+		}
+		for (i3=2*sx3; i3 < (sy - 2)*sx3; i3 += (sx3<<1)) {
+			for (j3=3; j3 < sx3; j3+=6) {        // G-points (2/2)
+				base=i3+j3;
+				tmp = outG[base] +
+					((outR[base - sx3] -
+					  outG[base - sx3] +
+					  outR[base + sx3] -
+					  outG[base + sx3]) >> 1);
+				CLIP(tmp, outR[base]);
+			}
+			for (j3=6; j3 < sx3 - 3; j3+=6) {        // B-points
+				base=i3+j3;
+				tmp = outG[base] +
+					((outR[base - sx3 - 3] -
+					  outG[base - sx3 - 3] +
+					  outR[base - sx3 + 3] -
+					  outG[base - sx3 + 3] +
+					  outR[base + sx3 - 3] -
+					  outG[base + sx3 - 3] +
+					  outR[base + sx3 + 3] -
+					  outG[base + sx3 + 3]) >> 2);
+				CLIP(tmp, outR[base]);
+			}
+		}
+
+		// process BLUE channel
+		for (i = 0,i3=0; i < sy*sx; i += (sx<<1), i3 += (sx3<<1)) {
+			for (j = 1, j3=3; j < sx - 2; j += 2, j3+=6) {
+				base=i3+j3;
+				tmp = outG[base] +
+					((outB[base - 3] -
+					  outG[base - 3] +
+					  outB[base + 3] -
+					  outG[base + 3]) >> 1);
+				CLIP(tmp, outB[base]);
+			}
+		}
+		for (i3=sx3; i3 < (sy - 1)*sx3; i3 += (sx3<<1)) {
+			for (j3=0; j3 < sx3 - 3; j3+=6) {
+				base=i3+j3;
+				tmp = outG[base] +
+					((outB[base - sx3] -
+					  outG[base - sx3] +
+					  outB[base + sx3] -
+					  outG[base + sx3]) >> 1);
+				CLIP(tmp, outB[base]);
+			}
+			for (j3=3; j3 < sx3 - 6; j3+=6) {
+				base=i3+j3;
+				tmp = outG[base] +
+					((outB[base - sx3 - 3] -
+					  outG[base - sx3 - 3] +
+					  outB[base - sx3 + 3] -
+					  outG[base - sx3 + 3] +
+					  outB[base + sx3 - 3] -
+					  outG[base + sx3 - 3] +
+					  outB[base + sx3 + 3] -
+					  outG[base + sx3 + 3]) >> 2);
+				CLIP(tmp, outB[base]);
+			}
+		}
+		break;
+    }
+
+    ClearBorders(rgb, sx, sy, 3);
+
+    return DC1394_SUCCESS;
 }
 
 /* coriander's Bayer decoding */
@@ -536,7 +801,7 @@ dc1394_bayer_Downsample(const uint8_t *restrict bayer, uint8_t *restrict rgb, in
             for (j = 0; j < sx; j += 2) {
                 tmp = ((bayer[i + j] + bayer[i + sx + j + 1]) >> 1);
                 CLIP(tmp, outG[((i >> 2) + (j >> 1)) * 3]);
-                tmp = bayer[i + sx + j + 1];
+                tmp = bayer[i + j + 1];
                 CLIP(tmp, outR[((i >> 2) + (j >> 1)) * 3]);
                 tmp = bayer[i + sx + j];
                 CLIP(tmp, outB[((i >> 2) + (j >> 1)) * 3]);
@@ -1048,8 +1313,273 @@ dc1394_bayer_HQLinear_uint16(const uint16_t *restrict bayer, uint16_t *restrict
 dc1394error_t
 dc1394_bayer_EdgeSense_uint16(const uint16_t *restrict bayer, uint16_t *restrict rgb, int sx, int sy, int tile, int bits)
 {
-    /* Removed due to patent concerns */
-    return DC1394_FUNCTION_NOT_SUPPORTED;
+	uint16_t *outR, *outG, *outB;
+    register int i3, j3, base;
+    int i, j;
+    int dh, dv;
+    int tmp;
+	int sx3=sx*3;
+
+    // sx and sy should be even
+    switch (tile) {
+    case DC1394_COLOR_FILTER_GRBG:
+    case DC1394_COLOR_FILTER_BGGR:
+        outR = &rgb[0];
+        outG = &rgb[1];
+        outB = &rgb[2];
+        break;
+    case DC1394_COLOR_FILTER_GBRG:
+    case DC1394_COLOR_FILTER_RGGB:
+        outR = &rgb[2];
+        outG = &rgb[1];
+        outB = &rgb[0];
+        break;
+    default:
+		return DC1394_INVALID_COLOR_FILTER;
+    }
+
+    switch (tile) {
+    case DC1394_COLOR_FILTER_GRBG:        //---------------------------------------------------------
+    case DC1394_COLOR_FILTER_GBRG:
+        // copy original RGB data to output images
+		for (i = 0, i3=0; i < sy*sx; i += (sx<<1), i3 += (sx3<<1)) {
+			for (j = 0, j3=0; j < sx; j += 2, j3+=6) {
+				base=i3+j3;
+				outG[base]           = bayer[i + j];
+				outG[base + sx3 + 3] = bayer[i + j + sx + 1];
+				outR[base + 3]       = bayer[i + j + 1];
+				outB[base + sx3]     = bayer[i + j + sx];
+			}
+		}
+		// process GREEN channel
+		for (i3= 3*sx3; i3 < (sy - 2)*sx3; i3 += (sx3<<1)) {
+			for (j3=6; j3 < sx3 - 9; j3+=6) {
+				base=i3+j3;
+				dh = abs(((outB[base - 6] +
+						   outB[base + 6]) >> 1) -
+						   outB[base]);
+				dv = abs(((outB[base - (sx3<<1)] +
+						   outB[base + (sx3<<1)]) >> 1) -
+						   outB[base]);
+				tmp = (((outG[base - 3]   + outG[base + 3]) >> 1) * (dh<=dv) +
+					   ((outG[base - sx3] + outG[base + sx3]) >> 1) * (dh>dv));
+				//tmp = (dh==dv) ? tmp>>1 : tmp;
+				CLIP16(tmp, outG[base], bits);
+			}
+		}
+		
+		for (i3=2*sx3; i3 < (sy - 3)*sx3; i3 += (sx3<<1)) {
+			for (j3=9; j3 < sx3 - 6; j3+=6) {
+				base=i3+j3;
+				dh = abs(((outR[base - 6] +
+						   outR[base + 6]) >>1 ) -
+						   outR[base]);
+				dv = abs(((outR[base - (sx3<<1)] +
+						   outR[base + (sx3<<1)]) >>1 ) -
+						   outR[base]);
+				tmp = (((outG[base - 3]   + outG[base + 3]) >> 1) * (dh<=dv) +
+					   ((outG[base - sx3] + outG[base + sx3]) >> 1) * (dh>dv));
+				//tmp = (dh==dv) ? tmp>>1 : tmp;
+				CLIP16(tmp, outG[base], bits);
+			}
+		}
+		// process RED channel
+		for (i3=0; i3 < (sy - 1)*sx3; i3 += (sx3<<1)) {
+			for (j3=6; j3 < sx3 - 3; j3+=6) {
+				base=i3+j3;
+				tmp = outG[base] +
+					((outR[base - 3] -
+					  outG[base - 3] +
+					  outR[base + 3] -
+					  outG[base + 3]) >> 1);
+				CLIP16(tmp, outR[base], bits);
+			}
+		}
+		for (i3=sx3; i3 < (sy - 2)*sx3; i3 += (sx3<<1)) {
+			for (j3=3; j3 < sx3; j3+=6) {
+				base=i3+j3;
+				tmp = outG[base] +
+					((outR[base - sx3] -
+					  outG[base - sx3] +
+					  outR[base + sx3] -
+					  outG[base + sx3]) >> 1);
+				CLIP16(tmp, outR[base], bits);
+			}
+			for (j3=6; j3 < sx3 - 3; j3+=6) {
+				base=i3+j3;
+				tmp = outG[base] +
+					((outR[base - sx3 - 3] -
+					  outG[base - sx3 - 3] +
+					  outR[base - sx3 + 3] -
+					  outG[base - sx3 + 3] +
+					  outR[base + sx3 - 3] -
+					  outG[base + sx3 - 3] +
+					  outR[base + sx3 + 3] -
+					  outG[base + sx3 + 3]) >> 2);
+				CLIP16(tmp, outR[base], bits);
+			}
+		}
+
+		// process BLUE channel
+		for (i3=sx3; i3 < sy*sx3; i3 += (sx3<<1)) {
+			for (j3=3; j3 < sx3 - 6; j3+=6) {
+				base=i3+j3;
+				tmp = outG[base] +
+					((outB[base - 3] -
+					  outG[base - 3] +
+					  outB[base + 3] -
+					  outG[base + 3]) >> 1);
+				CLIP16(tmp, outB[base], bits);
+			}
+		}
+		for (i3=2*sx3; i3 < (sy - 1)*sx3; i3 += (sx3<<1)) {
+			for (j3=0; j3 < sx3 - 3; j3+=6) {
+				base=i3+j3;
+				tmp = outG[base] +
+					((outB[base - sx3] -
+					  outG[base - sx3] +
+					  outB[base + sx3] -
+					  outG[base + sx3]) >> 1);
+				CLIP16(tmp, outB[base], bits);
+			}
+			for (j3=3; j3 < sx3 - 6; j3+=6) {
+				base=i3+j3;
+				tmp = outG[base] +
+					((outB[base - sx3 - 3] -
+					  outG[base - sx3 - 3] +
+					  outB[base - sx3 + 3] -
+					  outG[base - sx3 + 3] +
+					  outB[base + sx3 - 3] -
+					  outG[base + sx3 - 3] +
+					  outB[base + sx3 + 3] -
+					  outG[base + sx3 + 3]) >> 2);
+				CLIP16(tmp, outB[base], bits);
+			}
+		}
+		break;
+
+    case DC1394_COLOR_FILTER_BGGR:        //---------------------------------------------------------
+    case DC1394_COLOR_FILTER_RGGB:
+        // copy original RGB data to output images
+		for (i = 0, i3=0; i < sy*sx; i += (sx<<1), i3 += (sx3<<1)) {
+			for (j = 0, j3=0; j < sx; j += 2, j3+=6) {
+				base=i3+j3;
+				outB[base] = bayer[i + j];
+				outR[base + sx3 + 3] = bayer[i + sx + (j + 1)];
+				outG[base + 3] = bayer[i + j + 1];
+				outG[base + sx3] = bayer[i + sx + j];
+			}
+		}
+		// process GREEN channel
+		for (i3=2*sx3; i3 < (sy - 2)*sx3; i3 += (sx3<<1)) {
+			for (j3=6; j3 < sx3 - 9; j3+=6) {
+				base=i3+j3;
+				dh = abs(((outB[base - 6] +
+						   outB[base + 6]) >> 1) -
+						   outB[base]);
+				dv = abs(((outB[base - (sx3<<1)] +
+						   outB[base + (sx3<<1)]) >> 1) -
+						   outB[base]);
+				tmp = (((outG[base - 3]   + outG[base + 3]) >> 1) * (dh<=dv) +
+					   ((outG[base - sx3] + outG[base + sx3]) >> 1) * (dh>dv));
+				//tmp = (dh==dv) ? tmp>>1 : tmp;
+				CLIP16(tmp, outG[base], bits);
+			}
+		}
+		for (i3=3*sx3; i3 < (sy - 3)*sx3; i3 += (sx3<<1)) {
+			for (j3=9; j3 < sx3 - 6; j3+=6) {
+				base=i3+j3;
+				dh = abs(((outR[base - 6] +
+						   outR[base + 6]) >> 1) -
+						   outR[base]);
+				dv = abs(((outR[base - (sx3<<1)] +
+						   outR[base + (sx3<<1)]) >> 1) -
+						   outR[base]);
+				tmp = (((outG[base - 3]   + outG[base + 3]) >> 1) * (dh<=dv) +
+					   ((outG[base - sx3] + outG[base + sx3]) >> 1) * (dh>dv));
+				//tmp = (dh==dv) ? tmp>>1 : tmp;
+				CLIP16(tmp, outG[base], bits);
+			}
+		}
+		// process RED channel
+		for (i3=sx3; i3 < (sy - 1)*sx3; i3 += (sx3<<1)) {        // G-points (1/2)
+			for (j3=6; j3 < sx3 - 3; j3+=6) {
+				base=i3+j3;
+				tmp = outG[base] +
+					((outR[base - 3] -
+					  outG[base - 3] +
+					  outR[base + 3] -
+					  outG[base + 3]) >>1);
+				CLIP16(tmp, outR[base], bits);
+			}
+		}
+		for (i3=2*sx3; i3 < (sy - 2)*sx3; i3 += (sx3<<1)) {
+			for (j3=3; j3 < sx3; j3+=6) {        // G-points (2/2)
+				base=i3+j3;
+				tmp = outG[base] +
+					((outR[base - sx3] -
+					  outG[base - sx3] +
+					  outR[base + sx3] -
+					  outG[base + sx3]) >> 1);
+				CLIP16(tmp, outR[base], bits);
+			}
+			for (j3=6; j3 < sx3 - 3; j3+=6) {        // B-points
+				base=i3+j3;
+				tmp = outG[base] +
+					((outR[base - sx3 - 3] -
+					  outG[base - sx3 - 3] +
+					  outR[base - sx3 + 3] -
+					  outG[base - sx3 + 3] +
+					  outR[base + sx3 - 3] -
+					  outG[base + sx3 - 3] +
+					  outR[base + sx3 + 3] -
+					  outG[base + sx3 + 3]) >> 2);
+				CLIP16(tmp, outR[base], bits);
+			}
+		}
+
+		// process BLUE channel
+		for (i = 0,i3=0; i < sy*sx; i += (sx<<1), i3 += (sx3<<1)) {
+			for (j = 1, j3=3; j < sx - 2; j += 2, j3+=6) {
+				base=i3+j3;
+				tmp = outG[base] +
+					((outB[base - 3] -
+					  outG[base - 3] +
+					  outB[base + 3] -
+					  outG[base + 3]) >> 1);
+				CLIP16(tmp, outB[base], bits);
+			}
+		}
+		for (i3=sx3; i3 < (sy - 1)*sx3; i3 += (sx3<<1)) {
+			for (j3=0; j3 < sx3 - 3; j3+=6) {
+				base=i3+j3;
+				tmp = outG[base] +
+					((outB[base - sx3] -
+					  outG[base - sx3] +
+					  outB[base + sx3] -
+					  outG[base + sx3]) >> 1);
+				CLIP16(tmp, outB[base], bits);
+			}
+			for (j3=3; j3 < sx3 - 6; j3+=6) {
+				base=i3+j3;
+				tmp = outG[base] +
+					((outB[base - sx3 - 3] -
+					  outG[base - sx3 - 3] +
+					  outB[base - sx3 + 3] -
+					  outG[base - sx3 + 3] +
+					  outB[base + sx3 - 3] -
+					  outG[base + sx3 - 3] +
+					  outB[base + sx3 + 3] -
+					  outG[base + sx3 + 3]) >> 2);
+				CLIP16(tmp, outB[base], bits);
+			}
+		}
+		break;
+    }
+
+    ClearBorders_uint16(rgb, sx, sy, 3);
+
+    return DC1394_SUCCESS;
 }
 
 /* coriander's Bayer decoding */
@@ -2029,7 +2559,7 @@ dc1394_bayer_decoding_16bit(const uint16_t *restrict bayer, uint16_t *restrict r
 
 }
 
-void
+dc1394error_t
 Adapt_buffer_bayer(dc1394video_frame_t *in, dc1394video_frame_t *out, dc1394bayer_method_t method)
 {
     uint32_t bpp;
@@ -2051,7 +2581,8 @@ Adapt_buffer_bayer(dc1394video_frame_t *in, dc1394video_frame_t *out, dc1394baye
     }
 
     // the destination color coding is ALWAYS RGB. Set this.
-    if (in->color_coding==DC1394_COLOR_CODING_RAW16)
+    if ( (in->color_coding==DC1394_COLOR_CODING_RAW16) || 
+	 (in->color_coding==DC1394_COLOR_CODING_MONO16) )
         out->color_coding=DC1394_COLOR_CODING_RGB16;
     else
         out->color_coding=DC1394_COLOR_CODING_RGB8;
@@ -2062,7 +2593,8 @@ Adapt_buffer_bayer(dc1394video_frame_t *in, dc1394video_frame_t *out, dc1394baye
     // The output is never YUV, hence nothing to do about YUV byte order
 
     // bit depth is conserved for 16 bit and set to 8bit for 8bit:
-    if (in->color_coding==DC1394_COLOR_CODING_RAW16)
+    if ( (in->color_coding==DC1394_COLOR_CODING_RAW16) || 
+	 (in->color_coding==DC1394_COLOR_CODING_MONO16) )
         out->data_depth=in->data_depth;
     else
         out->data_depth=8;
@@ -2097,13 +2629,23 @@ Adapt_buffer_bayer(dc1394video_frame_t *in, dc1394video_frame_t *out, dc1394baye
     if (out->total_bytes>out->allocated_image_bytes) {
         free(out->image);
         out->image=(uint8_t*)malloc(out->total_bytes*sizeof(uint8_t));
+        if (out->image)
+            out->allocated_image_bytes = out->total_bytes*sizeof(uint8_t);
+        else
+            out->allocated_image_bytes = 0;
     }
 
     // Copy padding bytes:
-    memcpy(&(out->image[out->image_bytes]),&(in->image[in->image_bytes]),out->padding_bytes);
+    if(out->image)
+        memcpy(&(out->image[out->image_bytes]),&(in->image[in->image_bytes]),out->padding_bytes);
 
     out->little_endian=0; // not used before 1.32 is out.
     out->data_in_padding=0; // not used before 1.32 is out.
+    
+    if(out->image)
+        return DC1394_SUCCESS;
+        
+    return DC1394_MEMORY_ALLOCATION_FAILURE;
 }
 
 dc1394error_t
@@ -2115,7 +2657,10 @@ dc1394_debayer_frames(dc1394video_frame_t *in, dc1394video_frame_t *out, dc1394b
     switch (in->color_coding) {
     case DC1394_COLOR_CODING_RAW8:
     case DC1394_COLOR_CODING_MONO8:
-        Adapt_buffer_bayer(in,out,method);
+        
+        if(DC1394_SUCCESS != Adapt_buffer_bayer(in,out,method))
+            return DC1394_MEMORY_ALLOCATION_FAILURE;
+            
         switch (method) {
         case DC1394_BAYER_METHOD_NEAREST:
             return dc1394_bayer_NearestNeighbor(in->image, out->image, in->size[0], in->size[1], in->color_filter);
@@ -2137,7 +2682,10 @@ dc1394_debayer_frames(dc1394video_frame_t *in, dc1394video_frame_t *out, dc1394b
         break;
     case DC1394_COLOR_CODING_MONO16:
     case DC1394_COLOR_CODING_RAW16:
-        Adapt_buffer_bayer(in,out,method);
+    
+        if(DC1394_SUCCESS != Adapt_buffer_bayer(in,out,method))
+            return DC1394_MEMORY_ALLOCATION_FAILURE;
+            
         switch (method) {
         case DC1394_BAYER_METHOD_NEAREST:
             return dc1394_bayer_NearestNeighbor_uint16((uint16_t*)in->image, (uint16_t*)out->image, in->size[0], in->size[1], in->color_filter, in->data_depth);
