@@ -563,27 +563,27 @@ dc1394_convert_to_YUV422(uint8_t *src, uint8_t *dest, uint32_t width, uint32_t h
 {
     switch(source_coding) {
     case DC1394_COLOR_CODING_YUV422:
-        dc1394_YUV422_to_YUV422(src, dest, width, height, byte_order);
+        return dc1394_YUV422_to_YUV422(src, dest, width, height, byte_order);
         break;
     case DC1394_COLOR_CODING_YUV411:
-        dc1394_YUV411_to_YUV422(src, dest, width, height, byte_order);
+        return dc1394_YUV411_to_YUV422(src, dest, width, height, byte_order);
         break;
     case DC1394_COLOR_CODING_YUV444:
-        dc1394_YUV444_to_YUV422(src, dest, width, height, byte_order);
+        return dc1394_YUV444_to_YUV422(src, dest, width, height, byte_order);
         break;
     case DC1394_COLOR_CODING_RGB8:
-        dc1394_RGB8_to_YUV422(src, dest, width, height, byte_order);
+        return dc1394_RGB8_to_YUV422(src, dest, width, height, byte_order);
         break;
     case DC1394_COLOR_CODING_MONO8:
     case DC1394_COLOR_CODING_RAW8:
-        dc1394_MONO8_to_YUV422(src, dest, width, height, byte_order);
+        return dc1394_MONO8_to_YUV422(src, dest, width, height, byte_order);
         break;
     case DC1394_COLOR_CODING_MONO16:
     case DC1394_COLOR_CODING_RAW16:
-        dc1394_MONO16_to_YUV422(src, dest, width, height, byte_order, bits);
+        return dc1394_MONO16_to_YUV422(src, dest, width, height, byte_order, bits);
         break;
     case DC1394_COLOR_CODING_RGB16:
-        dc1394_RGB16_to_YUV422(src, dest, width, height, byte_order, bits);
+        return dc1394_RGB16_to_YUV422(src, dest, width, height, byte_order, bits);
         break;
     default:
         return DC1394_FUNCTION_NOT_SUPPORTED;
@@ -600,7 +600,7 @@ dc1394_convert_to_MONO8(uint8_t *restrict src, uint8_t *restrict dest, uint32_t
 {
     switch(source_coding) {
     case DC1394_COLOR_CODING_MONO16:
-        dc1394_MONO16_to_MONO8(src, dest, width, height, bits);
+        return dc1394_MONO16_to_MONO8(src, dest, width, height, bits);
         break;
     case DC1394_COLOR_CODING_MONO8:
         memcpy(dest, src, width*height);
@@ -619,24 +619,24 @@ dc1394_convert_to_RGB8(uint8_t *restrict src, uint8_t *restrict dest, uint32_t w
 {
     switch(source_coding) {
     case DC1394_COLOR_CODING_RGB16:
-        dc1394_RGB16_to_RGB8 (src, dest, width, height, bits);
+        return dc1394_RGB16_to_RGB8 (src, dest, width, height, bits);
         break;
     case DC1394_COLOR_CODING_YUV444:
-        dc1394_YUV444_to_RGB8 (src, dest, width, height);
+        return dc1394_YUV444_to_RGB8 (src, dest, width, height);
         break;
     case DC1394_COLOR_CODING_YUV422:
-        dc1394_YUV422_to_RGB8 (src, dest, width, height, byte_order);
+        return dc1394_YUV422_to_RGB8 (src, dest, width, height, byte_order);
         break;
     case DC1394_COLOR_CODING_YUV411:
-        dc1394_YUV411_to_RGB8 (src, dest, width, height);
+        return dc1394_YUV411_to_RGB8 (src, dest, width, height);
         break;
     case DC1394_COLOR_CODING_MONO8:
     case DC1394_COLOR_CODING_RAW8:
-        dc1394_MONO8_to_RGB8 (src, dest, width, height);
+        return dc1394_MONO8_to_RGB8 (src, dest, width, height);
         break;
     case DC1394_COLOR_CODING_MONO16:
     case DC1394_COLOR_CODING_RAW16:
-        dc1394_MONO16_to_RGB8 (src, dest, width, height,bits);
+        return dc1394_MONO16_to_RGB8 (src, dest, width, height,bits);
         break;
     case DC1394_COLOR_CODING_RGB8:
       memcpy(dest, src, width*height*3);
@@ -648,7 +648,7 @@ dc1394_convert_to_RGB8(uint8_t *restrict src, uint8_t *restrict dest, uint32_t w
     return DC1394_SUCCESS;
 }
 
-void
+dc1394error_t
 Adapt_buffer_convert(dc1394video_frame_t *in, dc1394video_frame_t *out)
 {
     uint32_t bpp;
@@ -701,14 +701,23 @@ Adapt_buffer_convert(dc1394video_frame_t *in, dc1394video_frame_t *out)
     if (out->total_bytes>out->allocated_image_bytes) {
         free(out->image);
         out->image=(uint8_t*)malloc(out->total_bytes*sizeof(uint8_t));
-        out->allocated_image_bytes=out->total_bytes;
+        if (out->image)
+            out->allocated_image_bytes = out->total_bytes*sizeof(uint8_t);
+        else
+            out->allocated_image_bytes = 0;
     }
 
     // Copy padding bytes:
-    memcpy(&(out->image[out->image_bytes]),&(in->image[in->image_bytes]),out->padding_bytes);
+    if(out->image)
+        memcpy(&(out->image[out->image_bytes]),&(in->image[in->image_bytes]),out->padding_bytes);
 
     out->little_endian=0;   // not used before 1.32 is out.
     out->data_in_padding=0; // not used before 1.32 is out.
+    
+    if(out->image)
+        return DC1394_SUCCESS;
+        
+    return DC1394_MEMORY_ALLOCATION_FAILURE;
 }
 
 dc1394error_t
@@ -719,35 +728,63 @@ dc1394_convert_frames(dc1394video_frame_t *in, dc1394video_frame_t *out)
     case DC1394_COLOR_CODING_YUV422:
         switch(in->color_coding) {
         case DC1394_COLOR_CODING_YUV422:
-            Adapt_buffer_convert(in,out);
-            dc1394_YUV422_to_YUV422(in->image, out->image, in->size[0], in->size[1], out->yuv_byte_order);
+            
+            if(DC1394_SUCCESS != Adapt_buffer_convert(in,out))
+                return DC1394_MEMORY_ALLOCATION_FAILURE;
+                
+            return dc1394_YUV422_to_YUV422(in->image, out->image, in->size[0], in->size[1], out->yuv_byte_order);
             break;
+            
         case DC1394_COLOR_CODING_YUV411:
-            Adapt_buffer_convert(in,out);
-            dc1394_YUV411_to_YUV422(in->image, out->image, in->size[0], in->size[1], out->yuv_byte_order);
+
+            if(DC1394_SUCCESS != Adapt_buffer_convert(in,out))
+                return DC1394_MEMORY_ALLOCATION_FAILURE;
+                
+            return dc1394_YUV411_to_YUV422(in->image, out->image, in->size[0], in->size[1], out->yuv_byte_order);
             break;
+            
         case DC1394_COLOR_CODING_YUV444:
-            Adapt_buffer_convert(in,out);
-            dc1394_YUV444_to_YUV422(in->image, out->image, in->size[0], in->size[1], out->yuv_byte_order);
+
+            if(DC1394_SUCCESS != Adapt_buffer_convert(in,out))
+                return DC1394_MEMORY_ALLOCATION_FAILURE;
+                
+            return dc1394_YUV444_to_YUV422(in->image, out->image, in->size[0], in->size[1], out->yuv_byte_order);
             break;
+            
         case DC1394_COLOR_CODING_RGB8:
-            Adapt_buffer_convert(in,out);
-            dc1394_RGB8_to_YUV422(in->image, out->image, in->size[0], in->size[1], out->yuv_byte_order);
+
+            if(DC1394_SUCCESS != Adapt_buffer_convert(in,out))
+                return DC1394_MEMORY_ALLOCATION_FAILURE;
+                
+            return dc1394_RGB8_to_YUV422(in->image, out->image, in->size[0], in->size[1], out->yuv_byte_order);
             break;
+            
         case DC1394_COLOR_CODING_MONO8:
         case DC1394_COLOR_CODING_RAW8:
-            Adapt_buffer_convert(in,out);
-            dc1394_MONO8_to_YUV422(in->image, out->image, in->size[0], in->size[1], out->yuv_byte_order);
+
+            if(DC1394_SUCCESS != Adapt_buffer_convert(in,out))
+                return DC1394_MEMORY_ALLOCATION_FAILURE;
+                
+            return dc1394_MONO8_to_YUV422(in->image, out->image, in->size[0], in->size[1], out->yuv_byte_order);
             break;
+            
         case DC1394_COLOR_CODING_MONO16:
         case DC1394_COLOR_CODING_RAW16:
-            Adapt_buffer_convert(in,out);
-            dc1394_MONO16_to_YUV422(in->image, out->image, in->size[0], in->size[1], out->yuv_byte_order, in->data_depth);
+
+            if(DC1394_SUCCESS != Adapt_buffer_convert(in,out))
+                return DC1394_MEMORY_ALLOCATION_FAILURE;
+                
+            return dc1394_MONO16_to_YUV422(in->image, out->image, in->size[0], in->size[1], out->yuv_byte_order, in->data_depth);
             break;
+            
         case DC1394_COLOR_CODING_RGB16:
-            Adapt_buffer_convert(in,out);
-            dc1394_RGB16_to_YUV422(in->image, out->image, in->size[0], in->size[1], out->yuv_byte_order, in->data_depth);
+
+            if(DC1394_SUCCESS != Adapt_buffer_convert(in,out))
+                return DC1394_MEMORY_ALLOCATION_FAILURE;
+                
+            return dc1394_RGB16_to_YUV422(in->image, out->image, in->size[0], in->size[1], out->yuv_byte_order, in->data_depth);
             break;
+            
         default:
             return DC1394_FUNCTION_NOT_SUPPORTED;
         }
@@ -755,13 +792,21 @@ dc1394_convert_frames(dc1394video_frame_t *in, dc1394video_frame_t *out)
     case DC1394_COLOR_CODING_MONO8:
         switch(in->color_coding) {
         case DC1394_COLOR_CODING_MONO16:
-            Adapt_buffer_convert(in,out);
-            dc1394_MONO16_to_MONO8(in->image, out->image, in->size[0], in->size[1], in->data_depth);
+
+            if(DC1394_SUCCESS != Adapt_buffer_convert(in,out))
+                return DC1394_MEMORY_ALLOCATION_FAILURE;
+                
+            return dc1394_MONO16_to_MONO8(in->image, out->image, in->size[0], in->size[1], in->data_depth);
             break;
+            
         case DC1394_COLOR_CODING_MONO8:
-            Adapt_buffer_convert(in,out);
+
+            if(DC1394_SUCCESS != Adapt_buffer_convert(in,out))
+                return DC1394_MEMORY_ALLOCATION_FAILURE;
+                
             memcpy(out->image, in->image, in->size[0]*in->size[1]);
             break;
+            
         default:
             return DC1394_FUNCTION_NOT_SUPPORTED;
         }
@@ -769,35 +814,63 @@ dc1394_convert_frames(dc1394video_frame_t *in, dc1394video_frame_t *out)
     case DC1394_COLOR_CODING_RGB8:
         switch(in->color_coding) {
         case DC1394_COLOR_CODING_RGB16:
-            Adapt_buffer_convert(in,out);
-            dc1394_RGB16_to_RGB8 (in->image, out->image, in->size[0], in->size[1], in->data_depth);
+
+            if(DC1394_SUCCESS != Adapt_buffer_convert(in,out))
+                return DC1394_MEMORY_ALLOCATION_FAILURE;
+                
+            return dc1394_RGB16_to_RGB8 (in->image, out->image, in->size[0], in->size[1], in->data_depth);
             break;
+            
         case DC1394_COLOR_CODING_YUV444:
-            Adapt_buffer_convert(in,out);
-            dc1394_YUV444_to_RGB8 (in->image, out->image, in->size[0], in->size[1]);
+
+            if(DC1394_SUCCESS != Adapt_buffer_convert(in,out))
+                return DC1394_MEMORY_ALLOCATION_FAILURE;
+                
+            return dc1394_YUV444_to_RGB8 (in->image, out->image, in->size[0], in->size[1]);
             break;
+            
         case DC1394_COLOR_CODING_YUV422:
-            Adapt_buffer_convert(in,out);
-            dc1394_YUV422_to_RGB8 (in->image, out->image, in->size[0], in->size[1], in->yuv_byte_order);
+
+            if(DC1394_SUCCESS != Adapt_buffer_convert(in,out))
+                return DC1394_MEMORY_ALLOCATION_FAILURE;
+                
+            return dc1394_YUV422_to_RGB8 (in->image, out->image, in->size[0], in->size[1], in->yuv_byte_order);
             break;
+            
         case DC1394_COLOR_CODING_YUV411:
-            Adapt_buffer_convert(in,out);
-            dc1394_YUV411_to_RGB8 (in->image, out->image, in->size[0], in->size[1]);
+
+            if(DC1394_SUCCESS != Adapt_buffer_convert(in,out))
+                return DC1394_MEMORY_ALLOCATION_FAILURE;
+                
+            return dc1394_YUV411_to_RGB8 (in->image, out->image, in->size[0], in->size[1]);
             break;
+            
         case DC1394_COLOR_CODING_MONO8:
         case DC1394_COLOR_CODING_RAW8:
-            Adapt_buffer_convert(in,out);
-            dc1394_MONO8_to_RGB8 (in->image, out->image, in->size[0], in->size[1]);
+
+            if(DC1394_SUCCESS != Adapt_buffer_convert(in,out))
+                return DC1394_MEMORY_ALLOCATION_FAILURE;
+                
+            return dc1394_MONO8_to_RGB8 (in->image, out->image, in->size[0], in->size[1]);
             break;
+            
         case DC1394_COLOR_CODING_MONO16:
         case DC1394_COLOR_CODING_RAW16:
-            Adapt_buffer_convert(in,out);
-            dc1394_MONO16_to_RGB8 (in->image, out->image, in->size[0], in->size[1],in->data_depth);
+
+            if(DC1394_SUCCESS != Adapt_buffer_convert(in,out))
+                return DC1394_MEMORY_ALLOCATION_FAILURE;
+                
+            return dc1394_MONO16_to_RGB8 (in->image, out->image, in->size[0], in->size[1],in->data_depth);
             break;
+            
         case DC1394_COLOR_CODING_RGB8:
-            Adapt_buffer_convert(in,out);
+
+            if(DC1394_SUCCESS != Adapt_buffer_convert(in,out))
+                return DC1394_MEMORY_ALLOCATION_FAILURE;
+                
             memcpy(out->image, in->image, in->size[0]*in->size[1]*3);
             break;
+            
         default:
             return DC1394_FUNCTION_NOT_SUPPORTED;
         }
@@ -874,16 +947,23 @@ Adapt_buffer_stereo(dc1394video_frame_t *in, dc1394video_frame_t *out)
     if (out->total_bytes>out->allocated_image_bytes) {
         free(out->image);
         out->image=(uint8_t*)malloc(out->total_bytes*sizeof(uint8_t));
-        out->allocated_image_bytes=out->total_bytes;
+        if (out->image)
+            out->allocated_image_bytes = out->total_bytes*sizeof(uint8_t);
+        else
+            out->allocated_image_bytes = 0;
     }
 
     // Copy padding bytes:
-    memcpy(&(out->image[out->image_bytes]),&(in->image[in->image_bytes]),out->padding_bytes);
+    if(out->image)
+        memcpy(&(out->image[out->image_bytes]),&(in->image[in->image_bytes]),out->padding_bytes);
 
     out->little_endian=0;   // not used before 1.32 is out.
     out->data_in_padding=0; // not used before 1.32 is out.
 
-    return DC1394_SUCCESS;
+    if(out->image)
+        return DC1394_SUCCESS;
+        
+    return DC1394_MEMORY_ALLOCATION_FAILURE;
 
 }
 
@@ -898,14 +978,24 @@ dc1394_deinterlace_stereo_frames(dc1394video_frame_t *in, dc1394video_frame_t *o
         switch (method) {
         case DC1394_STEREO_METHOD_INTERLACED:
             err=Adapt_buffer_stereo(in,out);
-            dc1394_deinterlace_stereo(in->image, out->image, in->size[0], in->size[1]);
+            if(err != DC1394_SUCCESS)
+                return err;
+                
+            return dc1394_deinterlace_stereo(in->image, out->image, out->size[0], out->size[1]);
             break;
+            
         case DC1394_STEREO_METHOD_FIELD:
             err=Adapt_buffer_stereo(in,out);
+            if (err != DC1394_SUCCESS)
+                return err;
+                
             memcpy(out->image,in->image,out->image_bytes);
-        break;
+	    return DC1394_SUCCESS;
+            break;
+        default:
+            return DC1394_INVALID_STEREO_METHOD;
         }
-        return DC1394_INVALID_STEREO_METHOD;
+// Never reached:  return DC1394_INVALID_STEREO_METHOD;
     }
     else
         return DC1394_FUNCTION_NOT_SUPPORTED;
